/*---------------------------------------------------------------
	FILE: CObservationVisualLandmarks.h
	USE: See doc below

   Part of the MRPT Library
   ISA - Universidad de Malaga - http://www.isa.uma.es
  ---------------------------------------------------------------*/
#ifndef CObservationVisualLandmarks_H
#define CObservationVisualLandmarks_H

#include <MRPT/UTILS/CSerializable.h>
#include <MRPT/MRML/CObservation.h>
#include <MRPT/MRML/CPose3D.h>
#include <MRPT/MRML/CPose2D.h>
#include <MRPT/MRML/CLandmarksMap.h>

namespace MRML
{

	/** Declares a class derived from "CObservation" that stores a Landmarks Map as seen from a stereo camera at a given instant of time.
	 *
	 * \sa CLandmarksMap, CObservation
	 */
	class CObservationVisualLandmarks : public CObservation
	{
		// This must be added to any CSerializable derived class:
		DEFINE_SERIALIZABLE( CObservationVisualLandmarks )

	 public:
		/** Constructor
		 */
		CObservationVisualLandmarks( );

		/** The 3D pose of the reference camera relative to robot coordinates.
		  */
		CPose3D				refCameraPose;

		 /** The landmarks, with coordinates origin in the camera reference system.
		  */
		CLandmarksMap		landmarks;

		/** Implements the virtual method in charge of finding the likelihood between this
		  *   and another observation, probably only of the same derived class. The operator
		  *   may be asymmetric.
		  *
		  * \param anotherObs The other observation to compute likelihood with.
		  * \param anotherObsPose If known, the belief about the robot pose when the other observation was taken can be supplied here, or NULL if it is unknown.
		  *
		  * \return Returns a likelihood measurement, in the range [0,1].
		  *	\exception std::exception On any error, as another observation being of an invalid class.
		  */
		 float  likelihoodWith( CObservation *anotherObs, CPosePDF *anotherObsPose = NULL );

	}; // End of class def.

} // End of namespace

#endif
