/*---------------------------------------------------------------
	FILE: CLoadableOptions.h
	USE: See description below.

   Part of the MRPT Library
   ISA - Universidad de Malaga - http://www.isa.uma.es
  ---------------------------------------------------------------*/
#ifndef  CLoadableOptions_H
#define  CLoadableOptions_H

#include <MRPT/UTILS/utils_defs.h>
#include <MRPT/UTILS/CStdOutStream.h>
#include <MRPT/UTILS/CConfigFileBase.h>
#include <MRPT/UTILS/MRPT_OS.h>

/*---------------------------------------------------------------
	Class
  ---------------------------------------------------------------*/
namespace UTILS
{
	/** This is a virtual base class for sets of options than can be loaded from a ".ini" file
	 */
	class CLoadableOptions
	{
	public:
		/** This method load the options from a ".ini"-like file or memory-stored string list.
		 *   Only those parameters found in the given "section" and having
		 *   the same name that the variable are loaded. Those not found in
		 *   the file will stay with their previous values (usually the default
		 *   values loaded at initialization). An example of an ".ini" file:
		 *  \code
		 *  [section]
		 *	resolution=0.10		; blah blah...
		 *	modeSelection=1		; 0=blah, 1=blah,...
		 *  \endcode
		 */
		virtual void  loadFromConfigFile(
			const CConfigFileBase	&source,
			const std::string		&section) = 0;

		/** This method must display clearly all the contents of the structure in textual form, sending it to a CStream.
		  */
		void  dumpToConsole()
		{
			MRPT_TRY_START;
			dumpToTextStream( UTILS::stdOut );
			MRPT_TRY_END;
		}

		/** This method must display clearly all the contents of the structure in textual form, sending it to a CStream.
		  */
		virtual void  dumpToTextStream(
			CStream		&out) = 0;

        /** Virtual destructor
          */
        virtual ~CLoadableOptions()
		{
		}

	}; // End of class def.

} // End of namespace

#endif
